'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports.Configuration = void 0;

var _friendlyErrorsWebpackPlugin = _interopRequireDefault(require('friendly-errors-webpack-plugin'));

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {default: obj};
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);
    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(
        Object.getOwnPropertySymbols(source).filter(function(sym) {
          return Object.getOwnPropertyDescriptor(source, sym).enumerable;
        })
      );
    }
    ownKeys.forEach(function(key) {
      _defineProperty(target, key, source[key]);
    });
  }
  return target;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {value: value, enumerable: true, configurable: true, writable: true});
  } else {
    obj[key] = value;
  }
  return obj;
}

class Configuration {
  constructor(config, mode, options) {
    this.config = config;
    this.mode = mode;
    this.options = options;
    this.config.addPlugin(new _friendlyErrorsWebpackPlugin.default());
  }

  entry() {
    if (this.config.entries.size < 1) {
      throw new Error('No entry specified.');
    }

    const entry = {};
    this.config.entries.forEach((path, name) => {
      entry[name] = path;
    });
    return entry;
  }

  output() {
    if (!this.config.output) {
      throw new Error('No output specified.');
    }

    return this.config.output;
  }

  rules() {
    const loaders = [];
    this.config.loaders.forEach((loader) => {
      loaders.push(loader);
    });
    return loaders;
  }

  plugins() {
    const plugins = [];
    this.config.plugins.forEach((plugin, position) => {
      plugins.push(
        _objectSpread({}, plugin, {
          position
        })
      );
    });
    return plugins
      .sort((a, b) => {
        // Keep the original order if two plugins have the same priority
        if (a.priority === b.priority) {
          return (a.position = b.position);
        } // A plugin with a priority of -10 will be placed after one
        // that has a priority of 0.

        return b.priority - a.priority;
      })
      .map((plugin) => plugin.plugin);
  }

  extensions() {
    const extensions = [];
    this.config.extensions.forEach((extension) => {
      extensions.push(extension);
    });
    return extensions;
  }

  aliases() {
    const aliases = {};
    this.config.aliases.forEach((path, name) => {
      aliases[name] = path;
    });
    return aliases;
  }

  externals() {
    const externals = {};
    this.config.externals.forEach((path, name) => {
      externals[name] = path;
    });
    return externals;
  }

  modules() {
    const modules = [];
    this.config.resolvePaths.forEach((path) => {
      modules.push(path);
    });
    return modules;
  }

  build() {
    let config = {
      context: this.config.context,
      mode: this.mode || 'development',
      devtool: 'source-map',
      entry: this.entry(),
      output: this.output(),
      module: {
        rules: this.rules()
      },
      plugins: this.plugins(),
      performance: {
        hints: false
      },
      resolve: {
        extensions: this.extensions(),
        alias: this.aliases(),
        modules: this.modules()
      },
      externals: this.externals()
    };

    if (this.mode === 'development') {
      config.devtool = 'inline-source-map';
    }

    this.config.configs.forEach((callback) => {
      config = callback(config);
    });
    return config;
  }
}

exports.Configuration = Configuration;
