/// <reference types="webpack" />
declare module "plebpack" {
    import { Output as IWebpackOutputOptions, Plugin, RuleSetRule } from 'webpack';
    export interface IOutputOptions extends IWebpackOutputOptions {
        path: string;
        filename: string;
    }
    export interface IPlebpack {
        context: string;
        use(...hooks: Function[]): void;
        addEntry(name: string, path: string): void;
        setOutput(options: IOutputOptions): void;
        setContext(context: string): void;
        addAlias(alias: string, path: string): void;
        addPlugin(plugin: Plugin, priority: number): void;
        addLoader(loader: RuleSetRule): void;
        addExtension(extension: string): void;
        addExternal(name: string, path: string): void;
        addResolvePath(path: string): void;
        merge(config: object | Function): void;
    }
    export class Plebpack implements IPlebpack {
        pkg?: string | null | undefined;
        private hooks;
        context: string;
        entries: Map<string, string>;
        output?: IOutputOptions;
        aliases: Map<string, string>;
        plugins: Set<{
            plugin: Plugin;
            priority: number;
        }>;
        loaders: Set<RuleSetRule>;
        extensions: Set<string>;
        resolvePaths: Set<string>;
        externals: Map<string, string>;
        configs: Set<Function>;
        constructor(pkg?: string | null | undefined);
        use(...hooks: Function[]): void;
        addEntry(name: string, path: string): void;
        setOutput(output: IOutputOptions): void;
        setContext(context: string): void;
        addAlias(alias: string, path: string): void;
        addPlugin(plugin: Plugin, priority?: number): void;
        addLoader(loader: RuleSetRule): void;
        addExtension(extension: string): void;
        addResolvePath(path: string): void;
        addExternal(name: string, path: string): void;
        merge(config: object | Function): void;
        toConfig(mode: string, options: object): object;
    }
}
declare module "configuration" {
    import { Plebpack } from "plebpack";
    export class Configuration {
        protected config: Plebpack;
        protected mode: string;
        protected options: object;
        constructor(config: Plebpack, mode: string, options: object);
        private entry;
        private output;
        private rules;
        private plugins;
        private extensions;
        private aliases;
        private externals;
        private modules;
        build(): object;
    }
}
declare module "index" {
    const config: {
        (env?: string, options?: object): object;
        use: (...hooks: Function[]) => void;
    };
    export default config;
}
declare module "hooks/context" {
    const _default: (path: string) => Function;
    export default _default;
}
declare module "hooks/entry" {
    const _default_1: (name: string, path?: string | undefined) => Function;
    export default _default_1;
}
declare module "hooks/loader" {
    import { RuleSetRule } from 'webpack';
    const _default_2: (loader: RuleSetRule) => Function;
    export default _default_2;
}
declare module "hooks/output" {
    import { IOutputOptions } from "plebpack";
    const _default_3: (options: string | IOutputOptions) => Function;
    export default _default_3;
}
declare module "hooks/plugin" {
    import { Plugin } from 'webpack';
    const _default_4: (plugin: Plugin, priority?: number) => Function;
    export default _default_4;
}
