'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports.Plebpack = void 0;

var _lodash = _interopRequireDefault(require('lodash.flatten'));

var _configuration = require('./configuration');

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {default: obj};
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);
    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(
        Object.getOwnPropertySymbols(source).filter(function(sym) {
          return Object.getOwnPropertyDescriptor(source, sym).enumerable;
        })
      );
    }
    ownKeys.forEach(function(key) {
      _defineProperty(target, key, source[key]);
    });
  }
  return target;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {value: value, enumerable: true, configurable: true, writable: true});
  } else {
    obj[key] = value;
  }
  return obj;
}

class Plebpack {
  constructor(pkg) {
    this.pkg = pkg;

    _defineProperty(this, 'hooks', new Set());

    _defineProperty(this, 'context', process.cwd());

    _defineProperty(this, 'entries', new Map());

    _defineProperty(this, 'output', void 0);

    _defineProperty(this, 'aliases', new Map());

    _defineProperty(this, 'plugins', new Set());

    _defineProperty(this, 'loaders', new Set());

    _defineProperty(this, 'extensions', new Set());

    _defineProperty(this, 'resolvePaths', new Set());

    _defineProperty(this, 'externals', new Map());

    _defineProperty(this, 'configs', new Set());
  }

  use(...hooks) {
    (0, _lodash.default)(hooks).forEach((hook) => {
      this.hooks.add(hook);
    });
  }

  addEntry(name, path) {
    if (this.entries.has(name)) {
      throw new Error(`Duplicate name "${name}". Entries must be unique.`);
    }

    this.entries.set(name, path);
  }

  setOutput(output) {
    this.output = output;
  }

  setContext(context) {
    this.context = context;
  }

  addAlias(alias, path) {
    this.aliases.set(alias, path);
  }

  addPlugin(plugin, priority = 0) {
    this.plugins.add({
      plugin,
      priority
    });
  }

  addLoader(loader) {
    this.loaders.add(loader);
  }

  addExtension(extension) {
    this.extensions.add(extension);
  }

  addResolvePath(path) {
    this.resolvePaths.add(path);
  }

  addExternal(name, path) {
    if (this.externals.has(name)) {
      throw new Error(`Duplicate name "${name}". Externals must be unique.`);
    }

    this.externals.set(name, path);
  }

  merge(config) {
    if (typeof config === 'function') {
      this.configs.add(config);
    }

    this.configs.add((c) => _objectSpread({}, c, config));
  }

  toConfig(mode, options) {
    this.hooks.forEach((hook) => hook(this));
    return new _configuration.Configuration(this, mode, options).build();
  }
}

exports.Plebpack = Plebpack;
